/*-
 * s.to cell - GRASS program to populate a raster map with values from a site list
 * Copyright (C) 1995. Simon Cox, CSIRO.
 *
 * Author: Simon Cox (Simon.Cox@ned.dem.csiro.au)
 *         AGCRC
 *	   CSIRO Exploration & Mining
 *	   PO Box 437
 *	   Nedlands  WA  6009
 *	   Australia
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for non-commercial purposes is hereby granted. This 
 * software is provided "as is" without express or implied warranty.
 *
 * SIMON COX (SJDC) MAKES NO EXPRESS OR IMPLIED WARRANTIES
 * (INCLUDING BY WAY OF EXAMPLE, MERCHANTABILITY) WITH RESPECT TO ANY
 * ITEM, AND SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL
 * OR CONSEQUENTAL DAMAGES ARISING OUT OF THE POSSESSION OR USE OF
 * ANY SUCH ITEM. LICENSEE AND/OR USER AGREES TO INDEMNIFY AND HOLD
 * SJDC HARMLESS FROM ANY CLAIMS ARISING OUT OF THE USE OR POSSESSION 
 * OF SUCH ITEMS.
 *
 * Modification History:
 * <25 Jun 1995> - began coding (sjdc)
 * <06 Jan 1994> - announced version 0.1B on pasture.ecn.purdue.edu (jdm)
 * <21 Jan 1994> - changed wording on help screen. Revised to 0.2B (jdm)
 *
 */

#pragma ident "s.to.cell v 0.1 <25 Jun 1995>; Copyright (c) 1995. Simon Cox"

#include <math.h>
#include "gis.h"
#include "site.h"
#include "site_count.h"


int main (argc, argv)
  char **argv;
  int argc;
{
  char *siteslist, errmsg[256], *oraster, *fraster, *mapset;
  int count_sum, count_count, n, freq, verbose;
  FILE *fdsite;
  SITE_LIST site_list;
  char	*desc;
  int fdorast,fdfrast;	/* file descriptor for raster file is int */
  struct RCLIST *rc_count, *rc_sum, *add_to_rc(); 
  double	scale,north,east;
  LONG	xcat;
  CELL	ocell, fcell;	/* a couple of buffers */
  struct Cell_head window;
  struct
  {
    struct Option *input, *output, *frequency, *z;
  } parm;
  struct
  {
    struct Flag *q;
  } flag;

  G_gisinit (argv[0]);

  parm.input = G_define_option ();
  parm.input->key = "input";
  parm.input->type = TYPE_STRING;
  parm.input->required = YES;
  parm.input->description = "sites list defining sample points";
  parm.input->gisprompt = "old,site_lists,sites";

  parm.output = G_define_option ();
  parm.output->key = "output";
  parm.output->type = TYPE_STRING;
  parm.output->required = YES;
  parm.output->description = "raster file to store values";
  parm.output->gisprompt = "new,cell,raster";

  parm.frequency = G_define_option ();
  parm.frequency->key = "frequency";
  parm.frequency->type = TYPE_STRING;
  parm.frequency->required = NO;
  parm.frequency->description = "raster file to store site frequencies";
  parm.frequency->gisprompt = "new,cell,raster";

  parm.z = G_define_option ();
  parm.z->key = "z";
  parm.z->type = TYPE_DOUBLE;
  parm.z->required = NO;
  parm.z->description = "factor by which to multiply site values\n\t(remember: rasters may only store integers)";
  parm.z->options = NULL;

  flag.q = G_define_flag ();
  flag.q->key = 'q';
  flag.q->description = "Quiet";

  if (G_parser (argc, argv))
    exit (1);

/* assign command options to variables */

  siteslist=parm.input->answer;
  oraster=parm.output->answer;
  freq= (parm.frequency->answer == (char)NULL) ? 0 : 1;
  if( freq ) fraster=parm.frequency->answer;
  if( parm.z->answer == NULL )
     scale= 1.0;
  else
     sscanf (parm.z->answer,"%lf", &scale);
  if (!scale)
     scale= 1.0;
  verbose = (flag.q->answer == (char)NULL) ? 1 : 0;
  
/* set up GIS environment and files */

  G_get_window (&window);

/*  if( verbose )	printf("opening site list ...\n");*/

  if ((mapset = G_find_file ("site_lists", siteslist, "")) == NULL)
  {
    sprintf (errmsg, "sites file [%s] not found", siteslist);
    G_fatal_error (errmsg);
  }
  if ((fdsite = G_fopen_sites_old (siteslist, mapset)) == NULL)
  {
    sprintf (errmsg, "can't open sites file [%s]", siteslist);
    G_fatal_error (errmsg);
  }

/*  if( verbose )	printf("creating empty raster files ...\n");*/

  if (G_legal_filename (oraster) < 0)
  {
    sprintf (errmsg, "[%s] illegal name", oraster);
    G_fatal_error (errmsg);
  }
  if ((fdorast = G_open_cell_new_random (oraster)) < 0 )
  {
    sprintf (errmsg, "can't create raster file [%s]", oraster);
    G_fatal_error (errmsg);
  }

  if( freq ){
  if (G_legal_filename (fraster) < 0)
  {
    sprintf (errmsg, "[%s] illegal name", fraster);
    G_fatal_error (errmsg);
  }
  if ((fdfrast = G_open_cell_new_random (fraster)) < 0 )
  {
    sprintf (errmsg, "can't create raster file [%s]", fraster);
    G_fatal_error (errmsg);
  }
  }
    
/* read in site list */

/*  if( verbose )	printf("reading in site list\n");*/
  initialize_site_list (&site_list);
  if (!read_site_list (&site_list, fdsite))
  {
    sprintf (errmsg, "can't read sites file [%s]", siteslist);
    G_fatal_error (errmsg);
  }

/* intialise buffers */

  rc_sum = NULL;
  rc_count = NULL;
  count_sum = 0;
  count_count = 0;
  
/* process site list */

  if( verbose )
  {
     printf ("%d sites to be processed\n", count_sites (&site_list, 0));
     counter_reset ("transferring sites to raster file ...", 0);
     counter (0);
  }

    rewind_site_list (&site_list);
    for (n = 1; next_site (&site_list, &north, &east, &desc); n++)
    {
	xcat = 0;
	if( desc[0] == '#' )	sscanf (desc,"#"LDSET"", &xcat);
	else 			sscanf (desc,""LDSET"", &xcat);
 	rc_sum = add_to_rc(&window,north,east,(CELL)(scale*xcat),rc_sum,&count_sum);
  	rc_count = add_to_rc(&window,north,east,(CELL)1,rc_count,&count_count);
  	if( verbose ) counter (n);
    }
    if( verbose ) printf("\n");
    for (n=0;n<count_sum;n++)
    {
       ocell= rc_sum[n].val/rc_count[n].val;
       G_put_map_row_random (fdorast,&ocell,(int)rc_sum[n].row,(int)rc_sum[n].col,1);
       if( freq ) G_put_map_row_random (fdfrast,&rc_count[n].val,(int)rc_count[n].row,(int)rc_count[n].col,1);
    }
    G_close_cell (fdorast);
    if( freq ) G_close_cell (fdfrast);

    if(verbose) printf ("compressing raster files ...\n");

    sprintf (errmsg, "r.compress %s > /dev/null", oraster);
    system (errmsg);
    if(verbose) printf("\n<%s> raster file complete\n", oraster);
    if( freq )
    {
    sprintf (errmsg, "r.compress %s > /dev/null", fraster);
    system (errmsg);
    if(verbose) printf("\n<%s> raster file complete\n", fraster);
    }
    if(verbose) printf ("run r.support to create support files\n");
    
    exit (1);
}
